/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.datamodel;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import net.sberk.jsgal.datamodel.dist.JSGalDist;
import net.sberk.util.MiscUtil;
import net.sberk.util.XMLUtil;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

@SuppressWarnings("nls")
public class Gallery
{
	private final static String	GALLERY_SYSDTD	="jsgal.dtd";
	private final static String	DOCTYPE		="jsgal";
	private final static String	TITLE		="title";
	private final static String THUMBS		="thumbs";
	private final static String	ROWS		="rows";
	private final static String	COLUMNS		="columns";
	private final static String	PAGESINROW	="pagesinrow";
	private final static String	WIDTH		="width";
	private final static String	HEIGHT		="height";
	private final static String	IMAGES		="images";
	private final static String	IMAGESDIR	="imagesdir";
	private final static String	THUMBSDIR	="thumbsdir";
	private final static String PREVIEWSCALE	="previewScale";
	private final static String	TARGET		="target";
	private final static String DEFSCALE	="defscale";
	public final static String	SCALE_VERT	="vert";
	public final static String	SCALE_HORIZ	="horiz";
	public final static String	SCALE_BOTH	="both";
	private final static String	IMAGE		="image";
	private final static String	FILE		="file";
	private final static String	NAME		="name";
	private final static String	THUMB		="thumb";
	private final static String	SCALE		="scale";	
		
	private final static int	DIM_NULL			=-1;
	private final static int	DEF_THUMB_WIDTH		=200;
	private final static int	DEF_THUMB_HEIGHT	=150;
	private final static int	DEF_IMAGE_WIDTH		=800;
	private final static int	DEF_IMAGE_HEIGHT	=600;
	
	// the title of the gallery
	private String	title;
	
	// the info from the tumb section
	private int	thumbRows=0;
	private int	thumbCols=0;
	private int	pagesInRows=20;
	private int	thumbWidth=DEF_THUMB_WIDTH;
	private int	thumbHeight=DEF_THUMB_HEIGHT;
	
	// the info from the images section
	private String	imagesDir="";
	private String	thumbsDir="";
	private int		imageWidth=DEF_IMAGE_WIDTH;
	private int		imageHeight=DEF_IMAGE_HEIGHT;
	private float	previewScale=0.7f;
	private String	target=ConvUtils.NULL;
	private String	defscale=SCALE_VERT;
	
	private ArrayList<Image>	images;
	
	public Gallery(String title, String thumbsDir, String imagesDir)
	{
		this();
		
		this.title=title;
		this.thumbsDir=thumbsDir;
		this.imagesDir=imagesDir;
	}
	
	private Gallery()
	{
		images=new ArrayList<Image>();
	}
	
	public String getTitle()
	{
		return title;
	}
	
	public void setTitle(String title)
	{
		this.title=title;
	}
	
	public String getImageDir()
	{
		return imagesDir;
	}

	public void setImageDir(String imageDir)
	{
		this.imagesDir=imageDir;
	}

	public int getImageHeight()
	{
		return imageHeight;
	}

	public void setImageHeight(int imageHeight)
	{
		this.imageHeight=imageHeight;
	}

	public int getImageWidth()
	{
		return imageWidth;
	}

	public void setImageWidth(int imageWidth)
	{
		this.imageWidth=imageWidth;
	}

	public int getPagesInRows()
	{
		return pagesInRows;
	}

	public void setPagesInRows(int pagesInRows)
	{
		this.pagesInRows=pagesInRows;
	}

	public float getPreviewScale() {
		return previewScale;
	}

	public void setPreviewScale(float previewScale)
	{
		this.previewScale = previewScale;
	}

	public String getTarget()
	{
		return target;
	}

	public void setTarget(String target)
	{
		this.target=target;
	}

	public String getDefScale() 
	{
		return defscale;
	}

	public void setDefScale(String defscale) 
	{
		this.defscale = defscale;
	}

	public int getThumbCols()
	{
		return thumbCols;
	}

	public void setThumbCols(int thumbCols)
	{
		this.thumbCols=thumbCols;
	}

	public int getThumbHeight()
	{
		return thumbHeight;
	}

	public void setThumbHeight(int thumbHeight)
	{
		this.thumbHeight=thumbHeight;
	}

	public int getThumbRows()
	{
		return thumbRows;
	}

	public void setThumbRows(int thumbRows)
	{
		this.thumbRows=thumbRows;
	}

	public String getThumbsDir()
	{
		return thumbsDir;
	}

	public void setThumbsDir(String thumbsDir)
	{
		this.thumbsDir=thumbsDir;
	}

	public int getThumbWidth()
	{
		return thumbWidth;
	}

	public void setThumbWidth(int thumbWidth)
	{
		this.thumbWidth=thumbWidth;
	}	
	
	public int size()
	{
		return images.size();
	}
	
	public int indexOf(Image img)
	{
		return images.indexOf(img);
	}
	
	public Image getImageAt(int index)
	{
		return images.get(index);
	}

	public void addImage(int index, Image img)
	{
		images.add(index, img);
		
	}
	public void addImage(Image img)
	{
		images.add(img);
	}
	
	public void removeImage(Image img)
	{
		images.remove(img);
	}
	
	public void removeImage(int index)
	{
		images.remove(index);
	}
	
	public static Gallery parseXML(File f) throws IOException, ParserConfigurationException, SAXException, IllegalArgumentException
	{
		Document	doc=XMLUtil.parseXML(f, true);
		Element		el=doc.getDocumentElement();
		
		if(!DOCTYPE.equals(el.getTagName()))
			throw new IllegalArgumentException("illegal doc type: "+el.getTagName());
		
		Gallery		gal=new Gallery();
		NodeList	nl=el.getChildNodes();
		
		for(int i=0; i<nl.getLength(); i++)
		{
			Node	nd=nl.item(i);
			
			if(nd instanceof Element)
			{
				Element	e=(Element)nd;
				
				if(TITLE.equals(e.getTagName()))
					gal.setTitle(XMLUtil.getElementText(e));
				else if(THUMBS.equals(e.getTagName()))
				{
					gal.thumbRows=ConvUtils.parseInteger(e.getAttribute(ROWS), gal.thumbRows);
					gal.thumbCols=ConvUtils.parseInteger(e.getAttribute(COLUMNS), gal.thumbCols);
					gal.pagesInRows=ConvUtils.parseInteger(e.getAttribute(PAGESINROW), gal.pagesInRows);
					gal.thumbWidth=ConvUtils.parseNullInteger(e.getAttribute(WIDTH), DIM_NULL);
					gal.thumbHeight=ConvUtils.parseNullInteger(e.getAttribute(HEIGHT), DIM_NULL);
					
					if(gal.thumbWidth<=0 && gal.thumbHeight<=0)
					{
						gal.thumbWidth=DEF_THUMB_WIDTH;
						gal.thumbHeight=DEF_THUMB_HEIGHT;
					}
					else if(gal.thumbWidth<=0)
						gal.thumbWidth=(gal.thumbHeight*4)/3;
					else if(gal.thumbHeight<=0)
						gal.thumbHeight=(gal.thumbWidth*3)/4;
				}
				else if(IMAGES.equals(e.getTagName()))
				{
					gal.imagesDir=e.getAttribute(IMAGESDIR);
					gal.thumbsDir=e.getAttribute(THUMBSDIR);
					gal.imageWidth=ConvUtils.parseNullInteger(e.getAttribute(WIDTH), DIM_NULL);
					gal.imageHeight=ConvUtils.parseNullInteger(e.getAttribute(HEIGHT), DIM_NULL);

					if(gal.imageWidth<=0 && gal.imageHeight<=0)
					{
						gal.imageWidth=DEF_IMAGE_WIDTH;
						gal.imageHeight=DEF_IMAGE_HEIGHT;
					}
					else if(gal.imageWidth<=0)
						gal.imageWidth=(gal.imageHeight*4)/3;
					else if(gal.thumbHeight<=0)
						gal.imageHeight=(gal.imageWidth*3)/4;
					
					gal.previewScale=ConvUtils.parseFloat(e.getAttribute(PREVIEWSCALE), gal.previewScale);
					gal.target=e.getAttribute(TARGET);
					gal.defscale=e.getAttribute(DEFSCALE);
					
					NodeList	nl2=e.getChildNodes();
					for(int j=0; j<nl2.getLength(); j++)
					{
						Node	nd2=nl2.item(j);
						
						if(nd2 instanceof Element)
						{
							Element	e2=(Element)nd2;
							
							if(IMAGE.equals(e2.getTagName()))
								gal.addImage(Image.parseDOM(e2));
							else
								throw new IllegalArgumentException("illegal tag: "+e2.getTagName());
						}
					}
				}
				else
					throw new IllegalArgumentException("illegal tag: "+e.getTagName());
			}
		}
		
		return gal;
	}
	
	public void writeXML(File gf) throws ParserConfigurationException, TransformerException, IOException
	{
		Document	doc=XMLUtil.createDOM();
	
		doc.appendChild(toDOM(doc));

		// first ensure the gallery folders exists
		if(thumbsDir!=null && thumbsDir.length()>0)
			MiscUtil.getRelativeFile(gf, thumbsDir).mkdirs();

		if(imagesDir!=null && imagesDir.length()>0)
			MiscUtil.getRelativeFile(gf, imagesDir).mkdirs();
		
		// finally write the gallery file
		XMLUtil.writeXML(doc, null, GALLERY_SYSDTD, new FileOutputStream(gf));
		
		// now also write the jsgal.dtd
		JSGalDist.writeJSGalDTD(gf);
	}
	
	Element toDOM(Document doc)
	{
		Element	el=doc.createElement(DOCTYPE);
		
		// the TITLE tag
		Element	e=doc.createElement(TITLE);
		if(title!=null && title.length()>0)
			e.appendChild(doc.createTextNode(title));
		el.appendChild(e);
		
		// the thumbs tag
		e=doc.createElement(THUMBS);
		e.setAttribute(ROWS, ConvUtils.int2String(thumbRows));
		e.setAttribute(COLUMNS, ConvUtils.int2String(thumbCols));
		e.setAttribute(PAGESINROW, ConvUtils.int2String(pagesInRows));
		e.setAttribute(WIDTH, ConvUtils.nullInt2String(thumbWidth));
		e.setAttribute(HEIGHT, ConvUtils.nullInt2String(thumbHeight));
		el.appendChild(e);
		
		// the images tag
		e=doc.createElement(IMAGES);
		if(imagesDir!=null && imagesDir.length()>0)
			e.setAttribute(IMAGESDIR, imagesDir);
		
		if(thumbsDir!=null && thumbsDir.length()>0)
			e.setAttribute(THUMBSDIR, thumbsDir);
		
		e.setAttribute(WIDTH, ConvUtils.nullInt2String(imageWidth));
		e.setAttribute(HEIGHT, ConvUtils.nullInt2String(imageHeight));
		e.setAttribute(PREVIEWSCALE, ConvUtils.float2String(previewScale));
		e.setAttribute(TARGET, target);
		if(defscale!=null && defscale.length()>0)
			e.setAttribute(DEFSCALE, defscale);
		
		el.appendChild(e);
		
		// the images
		for(int i=0; i<images.size(); i++)
			e.appendChild(images.get(i).toDOM(doc));
		
		return el;
	}
	
	public static class Image
	{
		private String	file;
		private String	name;
		private String	thumb;
		private String	descr;
		private String	scale;
		private Object	userData;
		
		public Image(String file, String name, String thumb, String scale, String descr)
		{
			this.file=file;
			this.name=name;
			if(thumb==null || thumb.length()<=0)
				this.thumb=file;
			else
				this.thumb=thumb;
			
			this.descr=descr;
			this.scale=scale;
		}

		public Object getUserData() 
		{
			return userData;
		}

		public void setUserData(Object userData)
		{
			this.userData = userData;
		}

		public String getDescr()
		{
			return descr;
		}

		public void setDescr(String descr)
		{
			this.descr=descr;
		}

		public String getFile()
		{
			return file;
		}

		public void setFile(String file)
		{
			this.file=file;
		}

		public String getName()
		{
			return name;
		}

		public void setName(String name)
		{
			this.name=name;
		}

		public String getThumb()
		{
			return thumb;
		}

		public void setThumb(String thumb)
		{
			this.thumb=thumb;
		}
		
		public String getScale() 
		{
			return scale;
		}

		public void setScale(String scale) 
		{
			this.scale = scale;
		}

		static Image parseDOM(Element el)
		{
			return new Image(el.getAttribute(FILE), el.getAttribute(NAME), el.getAttribute(THUMB), el.getAttribute(SCALE), XMLUtil.getElementText(el));
		}
		
		Element toDOM(Document doc)
		{
			Element		el=doc.createElement(IMAGE);
			
			el.setAttribute(FILE, file);
			if(name!=null && name.length()>0)
				el.setAttribute(NAME, name);
			
			// only write thumb if not equal to file
			if(thumb!=null && thumb.length()>0 && !MiscUtil.equals(file, thumb))
				el.setAttribute(THUMB, thumb);
			
			if(scale!=null && scale.length()>0)
				el.setAttribute(SCALE, scale);
			
			if(descr!=null && descr.length()>0)
				el.appendChild(doc.createTextNode(descr));
			
			return el;
		}
	}
}
