/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.Action;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.UIManager.LookAndFeelInfo;

import net.sberk.jsgal.gui.I18N.NamedAction;
import net.sberk.util.MiscUtil;

@SuppressWarnings({ "serial" })
class BuilderPrefsDialog extends JDialog implements ActionListener
{
	private JSGalBuilder	builder;
	
	private JComboBox	lafComboBox;
	private JCheckBox	sysDefBrowserBox;
	private JTextField	browserField;
	private JButton		browseButton;
	
	private Action	ACTION_BROWSE;
	private Action	ACTION_OKAY;
	private Action	ACTION_CANCEL;
	
	protected BuilderPrefsDialog(JSGalBuilder builder)
	{
		super(builder, I18N.PREFSDIALOG_TITLE, true);
	
		this.builder=builder;
		
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter()
		{
			@Override
			public void windowClosing(WindowEvent e)
			{
				cancelDialog();
			}		
		});

		initActions();
		
		getContentPane().setLayout(new BorderLayout(4, 4));
		
		JPanel	p1=new JPanel();
		p1.setLayout(new BoxLayout(p1, BoxLayout.Y_AXIS));

		p1.add(Box.createVerticalStrut(4));
		
		// create a look and feel selector
		LookAndFeelInfo	lafis[]=UIManager.getInstalledLookAndFeels();
		String	cLFN=UIManager.getLookAndFeel().getClass().getName();
		String	lafs[]=new String[lafis.length];
		String	cuLaF=null;
		for(int i=0; i<lafis.length; i++)
		{
			lafs[i]=lafis[i].getName();
			
			if(MiscUtil.equals(lafis[i].getClassName(), cLFN))
				cuLaF=lafis[i].getName();
		}
		
		lafComboBox=SwingHelper.addComboBox(p1, I18N.PREFSDIALOG_LOOKANDFEEL, lafs, cuLaF);
		// the browser selection
		sysDefBrowserBox=SwingHelper.addCheckBox(p1, I18N.PREFSDIALOG_USESYSBROWSER, BuilderPrefs.USE_SYSTEM_BROWSER, this);
		browseButton=new JButton(ACTION_BROWSE);
		browserField=SwingHelper.addTextField(p1, I18N.PREFSDIALOG_BROWSER, 40, BuilderPrefs.BROWSER_PATH, null, browseButton);
		getContentPane().add(p1, BorderLayout.CENTER);

		p1=new JPanel(new FlowLayout(FlowLayout.RIGHT));
		p1.add(new JButton(ACTION_OKAY));
		p1.add(new JButton(ACTION_CANCEL));
	
		getContentPane().add(p1, BorderLayout.SOUTH);

		// init controls and show dialog
		enableBrowserSelect();
		
		pack();
		setLocationRelativeTo(builder);
		setVisible(true);		
	}
	
	private void initActions()
	{
		ACTION_BROWSE=new NamedAction(I18N.PREFSDIALOG_BROWSE)
			{
				@Override
				public void actionPerformed(ActionEvent e) 
				{
					selectBrowser();
				}
			};
			
		ACTION_OKAY=new NamedAction(I18N.MESSAGE_OPT_OK)
			{
				@Override
				public void actionPerformed(ActionEvent e) 
				{
					okayDialog();
				}
			};

		ACTION_CANCEL=new NamedAction(I18N.MESSAGE_OPT_CANCEL)
			{
				@Override
				public void actionPerformed(ActionEvent e) 
				{
					cancelDialog();
				}
			};
	}

	private void enableBrowserSelect()
	{
		boolean enable=!sysDefBrowserBox.isSelected();
		
		browserField.setEnabled(enable);
		browseButton.setEnabled(enable);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		enableBrowserSelect();
	}

	private void applyChanges()
	{
		BuilderPrefs.USE_SYSTEM_BROWSER=sysDefBrowserBox.isSelected();
		BuilderPrefs.BROWSER_PATH=browserField.getText();
		
		String	laf=(String)lafComboBox.getSelectedItem();
		if(!MiscUtil.equals(laf, UIManager.getLookAndFeel().getName()))
		{
			for(LookAndFeelInfo lfi: UIManager.getInstalledLookAndFeels())
			{
				if(MiscUtil.equals(laf, lfi.getName()))
				{
					try
					{
						BuilderPrefs.setLookAndFeel(builder, lfi.getClassName());
					}
					catch(Exception e)
					{
						builder.reportProblem(e);
					}
					break;
				}
			}
		}
	}
	
	void selectBrowser()
	{
		File	f=new File(browserField.getText());
		
		f=builder.selectFile(f, null, I18N.PREFSDIALOG_SELBROWSER, I18N.FILE_CHOOSER_SELECT);
		
		if(f!=null)
			browserField.setText(f.getPath());
	}
	
	void okayDialog()
	{
		applyChanges();
		
		setVisible(false);
		dispose();		
	}
	
	void cancelDialog()
	{
		setVisible(false);
		dispose();
	}
}