/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.util.ArrayList;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.TransferHandler.DropLocation;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import net.sberk.jsgal.datamodel.Gallery.Image;
import net.sberk.jsgal.datamodel.GalleryCollection.GalleryEntry;
import net.sberk.jsgal.gui.GalleryPanel.ImageUserData;
import net.sberk.jsgal.gui.XHTMLPanel.GalleryEntryUserData;

class BuilderTreeHelper implements TreeModel, TreeCellRenderer, TreeSelectionListener, TreeWillExpandListener, MouseListener, SwingHelper.FileDropListener
{
	private JSGalBuilder	builder;
	private ArrayList<TreeModelListener>	listeners;
	private JPanel							lockedRenderComp;
	private DefaultTreeCellRenderer			dtcr;
	private	JLabel							lockedLbl;
	private Object							rootNode;
	
	protected BuilderTreeHelper(JSGalBuilder builder)
	{
		this.builder=builder;
		this.rootNode=new String("ROOT"); //$NON-NLS-1$
		this.listeners=new ArrayList<TreeModelListener>();
		
		this.lockedRenderComp=new JPanel(new BorderLayout());
		this.dtcr=new DefaultTreeCellRenderer();
		this.lockedLbl=new JLabel(I18N.ICON_LOCKED);
	}

	protected void projectChanged()
	{
		treeObjectChanged(rootNode, true);
	}
	
	protected void treeObjectChanged(Object obj, boolean structureChanged)
	{
		TreePath	tp=getPathForObject(obj);
		
		if(tp!=null)
		{
			TreeModelEvent	e=new TreeModelEvent(this, tp);
		
			if(structureChanged)
			{
				// the children have changed
				for(TreeModelListener l: listeners)
					l.treeStructureChanged(e);
			}
			else
			{
				// only the node itself has changed
				for(TreeModelListener l: listeners)
					l.treeNodesChanged(e);
			}
		}
	}
	
	protected void setSelectedObject(JTree tree, Object obj)
	{
		if(obj==null)
			tree.clearSelection();
		else if(tree.getSelectionPath()==null || obj!=tree.getSelectionPath().getLastPathComponent())
		{
			// we have to change the selection
			TreePath	tp=getPathForObject(obj);
			
			if(tp!=null)
				tree.setSelectionPath(tp);
		}
	}
	
	protected String getTooltipText(TreePath tp)
	{
		String	tip=null;
		if(tp!=null)
		{
			Object	o=tp.getLastPathComponent();
			
			if(o instanceof IModelPanel)
				tip=((IModelPanel)o).getTooltip();
			else if(o instanceof GalleryEntry)
			{
				GalleryEntryUserData	gud=(GalleryEntryUserData)((GalleryEntry)o).getUserData();
				tip=gud.getGalleryPath();
			}
			else if(o instanceof Image)
			{
				ImageUserData	iud=(ImageUserData)((Image)o).getUserData();
				tip=iud.getImagePath();
			}
		}
		
		return tip;
	}
	
	private TreePath getPathForObject(Object obj)
	{
		if(obj==rootNode)
			return new TreePath(rootNode);
		else if(obj instanceof XHTMLPanel)
			return new TreePath(new Object[] { rootNode, obj });
		else if(obj instanceof GalleryEntry)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)((GalleryEntry)obj).getUserData();
			
			return new TreePath(new Object[] { rootNode, gud.getXHTMLPanel(), obj });
		}
		else if(obj instanceof GalleryPanel)
		{
			GalleryEntry			ge=((GalleryPanel)obj).getGalleryEntry();
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			return new TreePath(new Object[] { rootNode, gud.getXHTMLPanel(), ge });
		}
		else if(obj instanceof Image)
		{
			ImageUserData			iud=(ImageUserData)((Image)obj).getUserData();
			
			if(iud!=null)
			{
				GalleryEntry			ge=iud.getGalleryPanel().getGalleryEntry();
				GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
				return new TreePath(new Object[] { rootNode, gud.getXHTMLPanel(), ge, obj });
			}
		}
		
		return null;
	}
	// the stuff for the model
	
	@Override
	public void addTreeModelListener(TreeModelListener l) 
	{
		listeners.add(l);
	}

	@Override
	public void removeTreeModelListener(TreeModelListener l) 
	{
		listeners.remove(l);
	}
	
	@Override
	public Object getChild(Object parent, int index) 
	{
		if(parent==rootNode && index==0)
			return builder.getProject();
		else if(parent instanceof XHTMLPanel)
			return ((XHTMLPanel)parent).getProjectPage().getGalCollection().getEntryAt(index);
		else if(parent instanceof GalleryEntry)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)((GalleryEntry)parent).getUserData();
			GalleryPanel			gp=gud.getGalleryPanel();
			
			if(gp!=null)
				return gp.getGallery().getImageAt(index);
		}
		
		return null;
	}

	@Override
	public int getChildCount(Object parent) 
	{
		if(parent==rootNode)
			return (builder.getProject()==null) ? 0 : 1;
		else if(parent instanceof XHTMLPanel)
			return ((XHTMLPanel)parent).getProjectPage().getGalCollection().size();
		else if(parent instanceof GalleryEntry)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)((GalleryEntry)parent).getUserData();
			GalleryPanel			gp=gud.getGalleryPanel();
			
			if(gp!=null)
				return gp.getGallery().size();
		}
		
		return 0;
	}

	@Override
	public int getIndexOfChild(Object parent, Object child) 
	{
		if(parent==rootNode && child==builder.getProject())
			return 0;
		else if(parent instanceof XHTMLPanel)
			return ((XHTMLPanel)parent).getProjectPage().getGalCollection().indexOf((GalleryEntry)child);
		else if(parent instanceof GalleryEntry)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)((GalleryEntry)parent).getUserData();
			GalleryPanel			gp=gud.getGalleryPanel();
			
			if(gp!=null)
				return gp.getGallery().indexOf((Image)child);
		}
		
		return 0;
	}

	@Override
	public Object getRoot() 
	{
		return rootNode;
	}

	@Override
	public boolean isLeaf(Object node) 
	{
		return (node instanceof Image);
	}

	@Override
	public void valueForPathChanged(TreePath path, Object newValue) 
	{
		// we don't support editing in the tree
	}

	// the stuff for the renderer
	@Override
	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean selected, boolean expanded, boolean leaf, int row,
			boolean hasFocus) 
	{
		Icon	icon=null;
		Object	rVal=value;
		boolean	locked=false;
		
		if(value instanceof IModelPanel)
		{
			rVal=((IModelPanel)value).getTitle();
			icon=((IModelPanel)value).getIcon();
		}
		else if(value instanceof GalleryEntry)
		{
			GalleryEntry			ge=(GalleryEntry)value;
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			rVal=ge.getFile();
			icon=I18N.ICON_GALLERY;
			locked=(gud.galleryPanel!=null);
		}
		else if(value instanceof Image)
		{
			rVal=((Image)value).getFile();
			icon=I18N.ICON_IMAGE;
		}

		dtcr.getTreeCellRendererComponent(tree, rVal, selected, expanded, leaf, row, hasFocus);
		dtcr.setIcon(icon);
		
		if(locked)
		{
			lockedRenderComp.add(dtcr, BorderLayout.CENTER);
			lockedRenderComp.add(lockedLbl, BorderLayout.EAST);
			
			return lockedRenderComp;
		}
		
		return dtcr;
	}
	
	// stuff for the TreeSelectionListener
	@Override
	public void valueChanged(TreeSelectionEvent e) 
	{
		Object		path[]=e.getPath().getPath();
		IModelPanel	imp=null;
		
		// find tab panel in path
		int	i=path.length-1;
		for(; i>=0; i--)
		{
			Object	o=path[i];
			
			if(o instanceof IModelPanel)
			{
				imp=(IModelPanel)o;
				break;
			}
			else if(o instanceof GalleryEntry)
			{
				GalleryEntryUserData	gud=(GalleryEntryUserData)((GalleryEntry)o).getUserData();

				if(gud.getGalleryPanel()!=null)
				{
					imp=gud.getGalleryPanel();
					break;
				}
			}
		}
		
		if(imp!=null)
		{
			Object	obj=imp;
			
			if(i<(path.length-1))
				obj=path[i+1];
			
			builder.setSelectedObject(imp, obj);
		}
	}

	// stuff for TreeWillExpand listener
	@Override
	public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException 
	{
		// nothing to do
	}

	@Override
	public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException
	{
		// if node is a GalleryEntry try to load the gallery (if not yet done)
		Object	obj=event.getPath().getLastPathComponent();
		
		if(obj instanceof GalleryEntry)
		{
			GalleryEntry			ge=(GalleryEntry)obj;
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			if(gud.galleryPanel==null)	// not yet loaded?
			{
				boolean	b=gud.getXHTMLPanel().openGallery(ge);
				
				if(!b)
					throw new ExpandVetoException(event);
			}
		}
	}

	// stuff for mouse listener
	@Override
	public void mouseClicked(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mouseEntered(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mouseExited(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
		checkForPopup(e);
	}

	@Override
	public void mouseReleased(MouseEvent e)	
	{
		checkForPopup(e);
	}

	private void checkForPopup(MouseEvent e)
	{
		if(e.isPopupTrigger() && e.getSource() instanceof JTree)
		{
			JTree	tree=(JTree)e.getSource();
			
			TreePath	tp=tree.getPathForLocation(e.getX(), e.getY());
			if(tp!=null)
			{
				Object		obj=tp.getLastPathComponent();
				IModelPanel	imp=null;
				
				if(obj instanceof XHTMLPanel)
					imp=(XHTMLPanel)obj;
				else if(obj instanceof GalleryEntry)
					imp=((GalleryEntryUserData)((GalleryEntry)obj).getUserData()).getXHTMLPanel();
				else if(obj instanceof Image)
					imp=((ImageUserData)((Image)obj).getUserData()).getGalleryPanel();

				if(imp!=null)
					imp.showPopup(tree, obj, e.getX(), e.getY());
			}
		}
	}

	
	// stuff for file drop listener
	
	@Override
	public boolean canImport(DropLocation location) 
	{
		Object	obj=getDropObject(location);

		// galleries or images are valid drop locations
		return (obj instanceof GalleryEntry) || (obj instanceof Image);
	}

	@Override
	public void filesDropped(DropLocation location, File[] files) 
	{
		Object			obj=getDropObject(location);
		GalleryPanel	gp=null;
		
		if(obj instanceof GalleryEntry)
		{
			GalleryEntry			ge=(GalleryEntry)obj;
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
		
			if(gud.galleryPanel==null)	// not yet loaded?
			{
				gud.getXHTMLPanel().openGallery(ge);
			}

			gp=gud.galleryPanel;
		}
		else if(obj instanceof Image)
			gp=((ImageUserData)((Image)obj).getUserData()).getGalleryPanel();
		
		if(gp!=null)
			gp.addImages(files);
	}
	
	private static Object getDropObject(DropLocation location)
	{
		if(location instanceof JTree.DropLocation)
		{
			JTree.DropLocation	treeDropLoc=(JTree.DropLocation)location;
			
			TreePath	path=treeDropLoc.getPath();
			if(path!=null)
				return path.getLastPathComponent();
		}
		
		return null;
	}
}