/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.Component;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

@SuppressWarnings("serial")
abstract class DefaultModelPanel extends JPanel implements DocumentListener, IModelPanel
{
	JSGalBuilder			builder;
	private Object			model;
	private JTabbedPane		parentPane;
	private boolean			isModified;
	
	protected DefaultModelPanel(JSGalBuilder builder, Object model, JTabbedPane parentPane)
	{
		this.builder=builder;
		this.model=model;
		this.parentPane=parentPane;
		
		setBorder(new EmptyBorder(4, 4, 4 ,4));
	}
	
	void addToTab()
	{
		// add to tab
		parentPane.insertTab(getTitle(), getIcon(), this, getTooltip(), parentPane.getTabCount());
	}
	
	abstract String _getTitle();
	
	void setModified(boolean modified)
	{
		isModified=modified;
			
		int	inx=parentPane.indexOfComponent(this);
		parentPane.setTitleAt(inx, getTitle());
		
		builder.treeObjectChanged(this, false);
	}

	public boolean isModified()
	{
		return isModified;
	}

	@Override
	public Component getComponent() 
	{
		return this;
	}

	@Override
	public String getTitle()
	{
		String	title=_getTitle();
		
		return isModified ? title+'*' : title;
	}
	
	@Override
	public Object getModel()
	{
		return model;
	}

	@Override
	public void changedUpdate(DocumentEvent e) 
	{
		setModified(true);
	}

	@Override
	public void insertUpdate(DocumentEvent e) 
	{
		setModified(true);
	}

	@Override
	public void removeUpdate(DocumentEvent e) 
	{
		setModified(true);
	}
}