/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.table.DefaultTableModel;

import net.sberk.jsgal.datamodel.Gallery;
import net.sberk.jsgal.gui.I18N.NamedAction;
import net.sberk.util.ExifExtractor;
import net.sberk.util.MiscUtil;
import net.sberk.util.ExifExtractor.ExifMetaData;

@SuppressWarnings("serial")
class ImagePreviewPanel extends JPanel implements Runnable
{
	private Dimension			previewImgDim;
	private JButton				imageButton;
	private DefaultTableModel	propsTableModel;
	private BufferedImage		image;
	private File				imageFile;
	private String				imageScale;
	private int					imageRotate;
	private Thread				imageThread=null;
	
	protected ImagePreviewPanel(Dimension previewImgDim)
	{
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createTitledBorder(I18N.IMGPREVIEW_TITLE));
		
		this.previewImgDim=previewImgDim;
		
		imageButton=new JButton();
		imageButton.setAlignmentX(0.5f);
		imageButton.setHorizontalAlignment(SwingConstants.CENTER);
		imageButton.setVerticalAlignment(SwingConstants.CENTER);
		imageButton.setToolTipText(I18N.IMGPREVIEW_BUTTON_TOOLTIP);
		imageButton.setBackground(Color.BLACK);
		imageButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) 
			{
				showImage();
			}
		});
		
		// set all sizes to preview dimensions
		imageButton.setMaximumSize(previewImgDim);
		imageButton.setMinimumSize(previewImgDim);
		imageButton.setPreferredSize(previewImgDim);
		add(imageButton);
		
		add(Box.createVerticalStrut(4));
		
		// now create the image prBufferedImage	operties table
		propsTableModel=new DefaultTableModel(I18N.IMGPREVIEW_TABLE_ROWS_LABELS, I18N.IMGPREVIEW_TABLE_HEADERS)
		{
			@Override
			public boolean isCellEditable(int row, int column) 
			{
				return false;
			}
		};
		
		JTable	jtab=new JTable(propsTableModel);
		jtab.getColumnModel().getColumn(0).setPreferredWidth(90);
		jtab.getColumnModel().getColumn(1).setPreferredWidth(140);
		add(jtab.getTableHeader());
		add(jtab);
	}
	
	void showImage()
	{
		if(image!=null)
		{
			new ImageDialog(SwingUtilities.windowForComponent(this), image, imageRotate, imageFile);
		}
	}
	
	protected void setNewImage(File newImgFile, String imgScale, int imgRotate)
	{
		synchronized(this)
		{
			if(!MiscUtil.equals(imageFile, newImgFile) || !MiscUtil.equals(imageScale, imgScale) || imgRotate!=imageRotate)
			{
				this.imageFile=newImgFile;
				this.imageScale=imgScale;
				this.imageRotate=imgRotate;
				this.image=null;
				
				// start a new image thread if not yet running
				if(imageThread==null)
				{
					imageThread=new Thread(this);
					imageThread.start();
				}
			}
		}
	}
	
	
	public void run()
	{
		String	s="dummy";	//$NON-NLS-1$ a dummy value
		File	f=new File(s); // a dummy file !=null !!!!!	
		int		rot=-1;
		
		for(;;)
		{
			synchronized(this)
			{
				if(MiscUtil.equals(f, imageFile) && MiscUtil.equals(s, imageScale) && rot==imageRotate)
				{
					// we're about to stop, so clear imageThread
					imageThread=null;
					break;
				}
				
				f=imageFile;
				s=imageScale;
				rot=imageRotate;
			}
			
			// we have a new image so let's first update the properties
			try
			{
				// set image props
				propsTableModel.setValueAt((f==null) ? null : f.getName(), I18N.IMGPREVIEW_TABLE_ROWS.FILE.ordinal(), 1);
				propsTableModel.setValueAt((f==null) ? null : new Long(f.length()), I18N.IMGPREVIEW_TABLE_ROWS.SIZE.ordinal(), 1);
				propsTableModel.setValueAt(null, I18N.IMGPREVIEW_TABLE_ROWS.ROTATION.ordinal(), 1);
				propsTableModel.setValueAt(null, I18N.IMGPREVIEW_TABLE_ROWS.DIM.ordinal(), 1);
				propsTableModel.setValueAt(null, I18N.IMGPREVIEW_TABLE_ROWS.TIME.ordinal(), 1);
				propsTableModel.setValueAt((f==null) ? I18N.IMGPREVIEW_STATUS_NOIMAGE : I18N.IMGPREVIEW_STATUS_LOADING, I18N.IMGPREVIEW_TABLE_ROWS.STATUS.ordinal(), 1);
				
				if(f==null)		// image was clears
				{
					imageButton.setIcon(null);
					continue;
				}
				
				// now load the image
				image=ImageIO.read(f);
				
				String	dim=image.getWidth()+"x"+image.getHeight(); //$NON-NLS-1$
				propsTableModel.setValueAt(dim, I18N.IMGPREVIEW_TABLE_ROWS.DIM.ordinal(), 1);

				
				synchronized(this)	// has the image changed?
				{
					if(!MiscUtil.equals(f, imageFile))
						continue;
				}

				// get the metadata timestamp
				ExifMetaData	md=ExifExtractor.getMetaData(f);
				propsTableModel.setValueAt(md.DateTime, I18N.IMGPREVIEW_TABLE_ROWS.TIME.ordinal(), 1);
				
				int	rotate=rot;
				if(rotate==ImageHelper.ROTATE_AUTO)
					rotate=ImageHelper.getRotationFromExif(md.Orientation);
				propsTableModel.setValueAt(I18N.ROTATE_DEGREES[rotate], I18N.IMGPREVIEW_TABLE_ROWS.ROTATION.ordinal(), 1);
				
				synchronized(this)	// has the image changed?
				{
					if(!MiscUtil.equals(f, imageFile))
						continue;
				}
				
				// create a scaled version and set in the imagelabel
				String	scale=s;
				if(scale==null)
					scale=ImageHelper.getScaling(image.getWidth(), image.getHeight(), previewImgDim.width, previewImgDim.height, Gallery.SCALE_VERT);

				Image	img=ImageHelper.getScaledImage(image, previewImgDim.width, previewImgDim.height, scale, false, rotate);
				imageButton.setIcon(new ImageIcon(img));

				synchronized(this)	// has the image changed?
				{
					if(!MiscUtil.equals(f, imageFile))
						continue;
				}
								
				// now create a smooth preview of the image if still no new image is requested
				img=ImageHelper.getScaledImage(image, previewImgDim.width, previewImgDim.height, scale, true, rotate);
				imageButton.setIcon(new ImageIcon(img));
				
				propsTableModel.setValueAt(I18N.IMGPREVIEW_STATUS_LOADED, I18N.IMGPREVIEW_TABLE_ROWS.STATUS.ordinal(), 1);
			}
			catch(Exception e)
			{
				propsTableModel.setValueAt(I18N.IMGPREVIEW_STATUS_LOAD_ERROR+e.getMessage(), I18N.IMGPREVIEW_TABLE_ROWS.STATUS.ordinal(), 1);
			}
		}
	}
}

@SuppressWarnings("serial")
class ImageDialog extends JDialog
{
	final static float	SCALEUP_FACTORS[]={ 1.5f, 2.0f, 3.0f, 4.0f };
	final static float	SCALEDOWN_FACTORS[]={ 0.75f, 0.5f, 0.375f, 0.25f, 0.188f, 0.125f, 0.094f, 0.062f};
	JScrollPane	scrollPane;
	String		titlePrefix;
	
	ImageDialog(Window parent, BufferedImage img, int rotate, File f)
	{
		super(parent, ModalityType.APPLICATION_MODAL);
				
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		
		if(img==null)
			return;
	
		titlePrefix=f.getName()+ " - "+img.getWidth()+"x"+img.getHeight()+" @ ";   //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		
		final ImageComponent	imagePanel=new ImageComponent(img);
		
		getContentPane().setLayout(new BorderLayout());
		
		JToolBar	tb=new JToolBar();
		tb.add(new NamedAction(I18N.IMGPREVIEW_ZOOM_IN)
			{
				@Override
				public void actionPerformed(ActionEvent e) { imagePanel.zoom(1); }
			});
		tb.add(new NamedAction(I18N.IMGPREVIEW_ZOOM_OUT)
			{
				@Override
				public void actionPerformed(ActionEvent e) { imagePanel.zoom(-1); }
			});
		tb.add(new NamedAction(I18N.IMGPREVIEW_ZOOM_ORIG)
			{
				@Override
				public void actionPerformed(ActionEvent e) { imagePanel.zoom(0); }
			});
		getContentPane().add(tb, BorderLayout.NORTH);
		
		scrollPane=new JScrollPane(imagePanel);
		scrollPane.setBackground(Color.BLACK);	
		getContentPane().add(scrollPane, BorderLayout.CENTER);
		
		pack();
		
		setLocationRelativeTo(parent);
		setVisible(true);
	}
	
	final static float getScaleFactor(int inx)
	{
		int		i;
		float	fs[];
		
		if(inx==0)
			return 1.0f;
		else if(inx>0)
		{
			i=inx-1;
			fs=SCALEUP_FACTORS;
		}
		else
		{
			i=-inx-1;
			fs=SCALEDOWN_FACTORS;
		}
			
		return (i<fs.length) ? fs[i] : fs[fs.length-1];
	}
	
	class ImageComponent extends JComponent implements MouseMotionListener, MouseListener, Runnable
	{
		private BufferedImage	origImage;
		private BufferedImage	scaledImage;
		private int				scaleIndex;
		private int				lastX;
		private int				lastY;
		private Thread			scalingThread;	
		
		ImageComponent(BufferedImage img)
		{
			this.origImage=img;
			this.scaleIndex=0;		// 1:1
		
			setAutoscrolls(true);
			addMouseListener(this);
			addMouseMotionListener(this);
			
			updateScaledImage(img);
		}

		@Override
		protected void paintComponent(Graphics g)
	    {
	    	// center image
	        int	x=(getWidth()-scaledImage.getWidth())/2;
	        int	y=(getHeight()-scaledImage.getHeight())/2;
	        
	        g.drawImage(scaledImage, x, y, this);
	    }
		
		private void updateScaledImage(BufferedImage bi)
		{
			Dimension	dim=new Dimension(bi.getWidth(), bi.getHeight());
			
			setPreferredSize(dim);
			setMaximumSize(dim);
			setMinimumSize(dim);
			
			scaledImage=bi;
			
			String	title=titlePrefix+(getScaleFactor(scaleIndex)*100.0f)+"%"; //$NON-NLS-1$
			setTitle(title);
			setToolTipText(title);
			
			revalidate();
			repaint();
		}

		@Override
		public void mouseClicked(MouseEvent e) { /* ignore */ }

		@Override
		public void mouseEntered(MouseEvent e) { /* ignore */ }

		@Override
		public void mouseExited(MouseEvent e) { /* ignore */ }

		@Override
		public void mousePressed(MouseEvent e) 
		{
			lastX=e.getXOnScreen();
			lastY=e.getYOnScreen();
			
			setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		}

		@Override
		public void mouseReleased(MouseEvent e) 
		{
			setCursor(null);
		}

		@Override
		public void mouseDragged(MouseEvent e)
		{
			Rectangle rect=getVisibleRect();

			int		x=e.getXOnScreen();
			int		y=e.getYOnScreen();
			
			// now scale offsets, so that the scroll is relative to scrollPane size
			int		w=scaledImage.getWidth();
			int		h=scaledImage.getHeight();

			float	fact=getScaleFactor(scaleIndex);
			
			int		xoff=4*(x-lastX)*(w-scrollPane.getWidth())/w;
			int		yoff=4*(y-lastY)*(h-scrollPane.getHeight())/h;
			
			xoff*=fact;
			yoff*=fact;
			
			rect.x+=xoff;
			rect.y+=yoff;
				
			scrollRectToVisible(rect);

			if(xoff!=0)
				lastX=x;
			if(yoff!=0)
				lastY=y;
		}

		@Override
		public void mouseMoved(MouseEvent e) 
		{
			// nothing to do
		}

		void zoom(int indexOff)
		{
			int		newInx;
			
			synchronized(this)
			{
				newInx=scaleIndex;
			}
			
			if(indexOff==0)
				newInx=0;
			else
				newInx+=indexOff;
				
			if(newInx>SCALEUP_FACTORS.length)
				newInx=SCALEUP_FACTORS.length;
			else if((-newInx)>SCALEDOWN_FACTORS.length)
				newInx=-SCALEDOWN_FACTORS.length;

			synchronized(this)
			{
				if(newInx!=scaleIndex)
				{
					scaleIndex=newInx;
					
					// now start scaling thread
					if(scalingThread==null)
					{
						scalingThread=new Thread(this);
						scalingThread.start();
					}
				}
			}
		}
		
		public void run()
		{
			int	factInx=SCALEUP_FACTORS.length+1;	// an invalid scale index !!!!
			
			for(;;)
			{
				synchronized(this)
				{
					if(factInx==scaleIndex)
					{
						// we're about to stop, so clear imageThread
						scalingThread=null;
						break;
					}
					
					factInx=scaleIndex;
				}

				if(factInx==0)
				{
					updateScaledImage(origImage);
					continue;
				}
				
				int		w=origImage.getWidth();
				float	fact=getScaleFactor(factInx);

				w=(int)(fact*w);
				
				// first do a quick scaling
				BufferedImage	bi=ImageHelper.getScaledImage(origImage, w, 0, Gallery.SCALE_HORIZ, false, ImageHelper.ROTATE_0);
					
				updateScaledImage(bi);
				
				synchronized(this)
				{
					// has the scaling changed?
					if(factInx!=scaleIndex)
						continue;
				}
				
				// now do a hq scaling
				bi=ImageHelper.getScaledImage(origImage, w, 0, Gallery.SCALE_HORIZ, true, ImageHelper.ROTATE_0);
				
				updateScaledImage(bi);
			}
		}
	}
}