/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;

import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.TreeSelectionModel;

import net.sberk.jsgal.datamodel.Gallery;
import net.sberk.jsgal.datamodel.XHTMLPage;
import net.sberk.jsgal.datamodel.GalleryCollection.GalleryEntry;
import net.sberk.jsgal.datamodel.dist.JSGalDist;
import net.sberk.jsgal.gui.XHTMLPanel.GalleryEntryUserData;
import net.sberk.util.MiscUtil;

@SuppressWarnings("serial")
public class JSGalBuilder extends JFrame implements ChangeListener, MouseListener
{
	private	BuilderActions		actions;
	
	private BuilderMenu			builderMenu;
	private JSplitPane			mainSplitPane;
	BuilderTreeHelper			treeHelper;
	private	JLabel				statusLabel;
	private JTree				projectTree;
	private JTabbedPane			tabPane;
	
	private XHTMLPanel			projPanel=null;
	private Hashtable<String, GalleryPanel>	galPanelMap;	
	
	private JSGalBuilder()
	{
		super(I18N.JSGB_TITLE);
	
		setIconImage(I18N.ICON_BUILDER.getImage());
		
		BuilderPrefs.loadPrefs(this);
		
		galPanelMap=new Hashtable<String, GalleryPanel>();
		
		actions=new BuilderActions(this);
		
		setLayout(new BorderLayout());
		builderMenu=new BuilderMenu(actions);
		setJMenuBar(builderMenu);
		
		setSize(BuilderPrefs.MAIN_WINDOW_RECT.width, BuilderPrefs.MAIN_WINDOW_RECT.height);
		
		if(BuilderPrefs.MAIN_WINDOW_RECT.x<0 || BuilderPrefs.MAIN_WINDOW_RECT.y<0)
			setLocationRelativeTo(null);
		else
			setLocation(BuilderPrefs.MAIN_WINDOW_RECT.x, BuilderPrefs.MAIN_WINDOW_RECT.y);

		addWindowListener(new JsgbWinAdapter());
		
		Container	c=getContentPane();
		c.add(new BuilderToolbar(actions), BorderLayout.NORTH);
		
		// now setup the content pane
		mainSplitPane=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		c.add(mainSplitPane, BorderLayout.CENTER);

		treeHelper=new BuilderTreeHelper(this);
		
		projectTree=new JTree(treeHelper)
		{
			@Override
			public String getToolTipText(MouseEvent evt) 
			{
				return treeHelper.getTooltipText(getPathForLocation(evt.getX(), evt.getY()));
			}			
		};
		projectTree.setCellRenderer(treeHelper);
		projectTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		projectTree.getSelectionModel().addTreeSelectionListener(treeHelper);
		projectTree.setToolTipText(I18N.EMPTY_STRING);	// not null to trigger tooltip rendering !!!
		projectTree.setRootVisible(false);
		projectTree.setExpandsSelectedPaths(true);
		projectTree.setShowsRootHandles(true);
		projectTree.addTreeWillExpandListener(treeHelper);
		SwingHelper.enableFileDrop(projectTree, treeHelper);
		
		mainSplitPane.setLeftComponent(new JScrollPane(projectTree));
		projectTree.addMouseListener(treeHelper);
		
		tabPane=new JTabbedPane();
		tabPane.addChangeListener(this);
		tabPane.addMouseListener(this);
		
		mainSplitPane.setRightComponent(tabPane);
		mainSplitPane.setDividerLocation(BuilderPrefs.MAIN_SPLIT_DIV_LOC);
		
		statusLabel=new JLabel();
		c.add(statusLabel, BorderLayout.SOUTH);
		
		updateProject(null);
	}
	
	protected BuilderActions getBuilderActions()
	{
		return actions;
	}
	
	protected XHTMLPage getProjectPage()
	{
		return (projPanel==null) ? null : projPanel.getProjectPage();
	}
	
	protected XHTMLPanel getProject()
	{
		return projPanel;
	}
	
	protected IModelPanel getSelectedModelPanel()
	{
		Component	comp=tabPane.getSelectedComponent();
		
		if(comp instanceof IModelPanel)
			return (IModelPanel)comp;
		
		return null;
	}
	
	private void updateProject(XHTMLPage pp)
	{
		if(projPanel!=null)
			tabPane.remove(projPanel);
		
		projPanel=(pp==null) ? null : new XHTMLPanel(this, pp, tabPane);

		// enable disable actions
		actions.projectUpdated();
		treeHelper.projectChanged();
		
		Object	obj=(projPanel==null) ? null : projPanel.getSelectedObject();
		setSelectedObject(projPanel, obj);
		
		if(pp!=null)
		{
			BuilderPrefs.addRecentProject(pp.getXhtmlFile());
			builderMenu.updateRecentFiles(actions);
		}
		
		if(pp!=null)
		{
			float	cvn=pp.getJSGalVersionNumber();
			float	dvn=JSGalDist.getJSGalVersionNumber();
			
			// show upgrade message if dist version is newer
			if(dvn>cvn)
			{
				String	msg=I18NMsg.MSG_UPGRADE_TO_NEWER_VERSION.fmt(pp.getJSGalVersion(), JSGalDist.getJSGalVersion());
				
				if(MessageDialog.showYesNoConfirm(this, msg))
					upgradeProject();
			}
		}
	}
	
	protected void removePanel(IModelPanel imp)
	{
		tabPane.remove(imp.getComponent());
		
		if(imp instanceof XHTMLPanel)
		{
			projPanel=null;	// since we already removed it
			updateProject(null);
		}
		else if(imp instanceof GalleryPanel)
		{
			// remove from map
			GalleryEntryUserData	gud=(GalleryEntryUserData)((GalleryPanel)imp).getGalleryEntry().getUserData();
			
			galPanelMap.remove(gud.getGalleryPath());
		}
	}
	
	private boolean selectionInProgress=false;
	protected void setSelectedObject(IModelPanel imp, Object obj)
	{
		// prevent recursive calls
		if(!selectionInProgress)
		{
			selectionInProgress=true;
			
			if(imp!=null)
			{
				// first update tab if needed
				int	inx=tabPane.indexOfComponent(imp.getComponent());
				if(inx>=0 && inx!=tabPane.getSelectedIndex())
				{
					tabPane.setSelectedIndex(inx);
				}
				
				if(imp.getSelectedObject()!=obj)
					imp.setSelectedObject(obj);
				else if(imp instanceof GalleryPanel)
				{
					// also select the gallery entry in project
					GalleryEntry			ge=((GalleryPanel)imp).getGalleryEntry();
					GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
					
					gud.getXHTMLPanel().setSelectedObject(ge);
				}
				
				// now update the tree
				treeHelper.setSelectedObject(projectTree, obj);
			}
			
			// now update actions
			actions.selectionUpdated();
			
			selectionInProgress=false;
		}
	}
	
	protected GalleryPanel addGalleryPanel(GalleryEntry ge, Gallery gal)
	{		
		try
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			GalleryPanel			gp=new GalleryPanel(this, gud.getGalleryFile().getCanonicalFile(), ge, gal, tabPane);
			
			galPanelMap.put(gud.getGalleryPath(), gp);
			
			return gp;
		}
		catch(IOException e)
		{
			reportProblem(e);
		}
		
		return null;
	}
	
	protected GalleryPanel findOpenGalleryPanelForFile(File f)
	{
		try
		{
			return galPanelMap.get(f.getCanonicalPath());
		}
		catch(IOException e)
		{
			reportProblem(e);
		}
		
		return null;
	}
	
	protected void treeObjectChanged(Object obj, boolean structureChanged)
	{
		treeHelper.treeObjectChanged(obj, structureChanged);
	}
	
	protected void exit()
	{
		if(closeAll())
		{
			// update window prefs
			getBounds(BuilderPrefs.MAIN_WINDOW_RECT);
			BuilderPrefs.MAIN_SPLIT_DIV_LOC=mainSplitPane.getDividerLocation();
			
			JSGalBuilder.this.setVisible(false);
			JSGalBuilder.this.dispose();
			
			BuilderPrefs.savePrefs();
			
			// finally exit jvm
			System.exit(0);
		}
	}
	
	protected void setStatus(String status)
	{
		statusLabel.setText(status);
	}
	
	protected void logProblem(String s)
	{
		System.err.println(s);
	}
	
	protected void reportProblem(Throwable t)
	{
		MessageDialog.showError(this, t.getMessage(), MiscUtil.getStacktrace(t));
	}

	protected void reportError(String msg)
	{
		MessageDialog.showError(this, msg, null);
	}
	
	protected void reportWarning(String msg)
	{
		MessageDialog.showWarning(this, msg, null);
	}
	
	protected void newProject()
	{
		JCheckBox	defBox=new JCheckBox(I18N.NEW_PROJECT_CREATEGAL, true);
		String	title=MessageDialog.getInput(this, I18N.NEW_PROJECT_TITLE, I18N.NEW_PROJECT_MSG, null, defBox);
		
		if(title==null || title.length()<=0)
			return;
		
		File	f=new File(MiscUtil.stringToFilename(title)+'.'+I18N.EXT_HTML);
		f=selectFile(f, I18N.XHTML_FILEFILTER, I18N.NEW_PROJECT_SAVETITLE, I18N.FILE_CHOOSER_SAVE);
		
		if(f!=null)
		{
			File galsFile=new File(f.getParentFile(), MiscUtil.changeFileNameExt(f.getName(), I18N.EXT_XML));
			
			try
			{
				XHTMLPage	xp=new XHTMLPage(f, title, galsFile, JSGalDist.getJSGalVersion());
				xp.create();
				
				updateProject(xp);
				
				if(defBox.isSelected())
				{
					// also create the default gallery
					projPanel.addNewGallery();
				}
			}
			catch(Exception e)
			{
				reportProblem(e);
			}
		}
	}
	
	protected void openProject()
	{
		File	f=selectFile(null, I18N.XHTML_FILEFILTER, I18N.OPEN_PROJECT_OPENTITLE, I18N.FILE_CHOOSER_OPEN);
		
		if(f!=null)
			loadProject(f);
	}
	
	protected void loadProject(File f)
	{
		try
		{
			XHTMLPage	xp=XHTMLPage.parseXHTML(f);
			
			updateProject(xp);
		}
		catch(Exception e)
		{
			reportProblem(e);
		}
	}
	
	protected void upgradeProject()
	{
		XHTMLPage	xp=getProjectPage();
		
		if(xp!=null)
		{
			float	cvn=xp.getJSGalVersionNumber();
			float	dvn=JSGalDist.getJSGalVersionNumber();
			
			// show warning if version newer
			if(cvn>dvn)
			{
				String	msg=I18NMsg.MSG_WARNING_UPGRADE_NEWER_VERSION.fmt(xp.getJSGalVersion(), JSGalDist.getJSGalVersion());
				boolean	res=MessageDialog.showYesNoWarning(this, msg);
				
				if(!res)
					return;
			}
			
			// show confirmation
			String	msg=I18NMsg.MSG_CONFIRM_UPGRADE.fmt(xp.getXhtmlFile().getPath());
			if(MessageDialog.showYesNoConfirm(this, msg))
			{
				try
				{
					xp.upgrade();
				}
				catch(Exception e)
				{
					reportProblem(e);
				}
			}
		}
	}
	
	protected void saveFile(IModelPanel	imp)
	{
		if(imp!=null)
			saveModelObject(imp);
	}
	
	protected void saveAll()
	{
		for(Component comp: tabPane.getComponents())
		{
			if(comp instanceof IModelPanel)
			{
				saveFile((IModelPanel)comp);
			}
		}
	}
	
	private void saveModelObject(IModelPanel imp)
	{
		try
		{
			imp.saveUpdatedModel();
		}
		catch(Exception e)
		{
			reportProblem(e);
		}
	}
	
	private JFileChooser	fileChooser;
	protected File selectFile(File selFile, FileFilter filter, String title, String[] okText)
	{
		if(fileChooser==null)
			fileChooser=new JFileChooser();

		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.resetChoosableFileFilters();
		
		if(selFile!=null && selFile.isDirectory())
		{
			fileChooser.setCurrentDirectory(selFile);
			fileChooser.setSelectedFile(null);
		}
		else
			fileChooser.setSelectedFile(selFile);
		
		if(filter!=null)
		{
			fileChooser.addChoosableFileFilter(filter);
			fileChooser.setFileFilter(filter);
		}
	
		fileChooser.setMultiSelectionEnabled(false);
		fileChooser.setDialogTitle(title);
		fileChooser.setApproveButtonText(okText[0]);
		fileChooser.setApproveButtonMnemonic(okText[1].charAt(0));
		fileChooser.setApproveButtonToolTipText(okText[2]);
		
		int rv=fileChooser.showDialog(this, null);

		return (rv==JFileChooser.APPROVE_OPTION) ? fileChooser.getSelectedFile() : null;
	}

	protected boolean closeFile(IModelPanel imp)
	{
		return imp.closePanel();
	}
	
	protected boolean closeAll()
	{
		boolean	ret=true;
		
		for(Component comp: tabPane.getComponents())
		{
			// closing the project panel will also take care of the opened galleries
			if(comp instanceof XHTMLPanel)
			{
				ret&=closeFile((XHTMLPanel)comp);
			}
		}
		
		return ret;
	}
	
	protected void startBrowser()
	{
		try
		{
			XHTMLPage	xp=getProjectPage();
			
			if(xp!=null)
			{
				BuilderPrefs.startBrowser(xp.getXhtmlFile().toURI());
			}
		}
		catch(Exception e)
		{
			reportProblem(e);
		}
	}
	
	protected void showPreferences()
	{
		new BuilderPrefsDialog(this);
	}
	
	public static void main(String args[])
	{
		JSGalBuilder	jsgb=new JSGalBuilder();
		
		jsgb.setVisible(true);
		
		if(args.length>0)
			jsgb.loadProject(new File(args[0]));
	}
	
	class JsgbWinAdapter extends WindowAdapter
	{
		@Override
		public void windowClosing(WindowEvent e) 
		{
			exit();
		}
	}
	
	// stuff for tabPange change listener
	public void stateChanged(ChangeEvent e) 
	{
		IModelPanel	imp=(IModelPanel)tabPane.getSelectedComponent();
		Object		obj=imp;
		
		if(imp!=null)
			obj=imp.getSelectedObject();
		
		setSelectedObject(imp, obj);
	}
	
	// stuff for mouse listener
	@Override
	public void mouseClicked(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mouseEntered(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mouseExited(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
		checkForPopup(e);
	}

	@Override
	public void mouseReleased(MouseEvent e)	
	{
		checkForPopup(e);
	}

	private void checkForPopup(MouseEvent e)
	{
		if(e.isPopupTrigger() && e.getSource()==tabPane)
		{
			int	inx=tabPane.getUI().tabForCoordinate(tabPane, e.getX(), e.getY());
			
			if(inx>=0 && inx<tabPane.getComponentCount())
			{
				Component	comp=tabPane.getComponent(inx);

				if(comp instanceof IModelPanel)
					((IModelPanel)comp).showPopup(tabPane, comp, e.getX(), e.getY());
			}
		}
	}	
}