/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.CellEditor;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.TransferHandler.DropLocation;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import net.sberk.jsgal.datamodel.Gallery;
import net.sberk.jsgal.datamodel.GalleryCollection;
import net.sberk.jsgal.datamodel.XHTMLPage;
import net.sberk.jsgal.datamodel.GalleryCollection.GalleryEntry;
import net.sberk.jsgal.gui.I18N.NamedAction;
import net.sberk.util.MiscUtil;

@SuppressWarnings("serial")
class XHTMLPanel extends DefaultModelPanel implements TableModel, ListSelectionListener, ActionListener, MouseListener, SwingHelper.FileDropListener
{
	private Action ACTION_DIALOG_GAL_MOVE_UP;
	private Action ACTION_DIALOG_GAL_MOVE_DOWN;
	private Action ACTION_DIALOG_GAL_OPEN;
	private Action ACTION_DIALOG_GAL_ADD_NEW;
	private Action ACTION_DIALOG_GAL_ADD_EXISTING;
	private Action ACTION_DIALOG_GAL_REMOVE;
	
	GalleryCollection	galCol;
	private ArrayList<TableModelListener>	tablstnrs;
	private JTextField	titleField;
	private JTextField	versionFileField;
	private JTextField	galleriesFileField;
	private JCheckBox	restoreGalBox;
	private JCheckBox	restoreImgBox;
	private JTextField	defImgField;
	private JComboBox	galOrientBox;
	private	JTextField	galsInHorizRowField;
	private JCheckBox	wrapGalBox;
	private JTable		galleryTable;
	
	protected XHTMLPanel(JSGalBuilder builder, XHTMLPage page, JTabbedPane parentPane)
	{
		super(builder, page, parentPane);
		
		this.galCol=page.getGalCollection();
	
		// create entry user data
		for(int i=galCol.size()-1; i>=0; i--)
		{
			new GalleryEntryUserData(galCol.getEntryAt(i), null);
		}
		
		initActions();
		
		tablstnrs=new ArrayList<TableModelListener>();
		
		setLayout(new BorderLayout(4, 4));
		
		JPanel	p1=new JPanel();
		p1.setLayout(new BoxLayout(p1, BoxLayout.Y_AXIS));
		
		titleField=SwingHelper.addTextField(p1, I18N.DIALOG_GALCOL_TITLE, 40, page.getTitle(), this);
		versionFileField=SwingHelper.addTextField(p1, I18N.DIALOG_GALCOL_JSGALVERSION, 20, page.getJSGalVersion(), this);
		versionFileField.setEnabled(false);
		
		String	path;
		try
		{
			path=MiscUtil.getRelativePath(page.getXhtmlFile(), page.getGalleriesFile());	
		}
		catch(Exception e)
		{
			path=page.getGalleriesFile().getPath();
		}
		galleriesFileField=SwingHelper.addTextField(p1, I18N.DIALOG_GALCOL_GALS_FILE, 40, path);
		galleriesFileField.setEnabled(false);
		galleriesFileField.setToolTipText(page.getGalleriesFile().getPath());

		p1.add(Box.createVerticalStrut(4));
		p1.add(new JSeparator(JSeparator.HORIZONTAL));
		p1.add(Box.createVerticalStrut(4));

		// the rendering options
		JPanel	p2=new JPanel();
		p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
		p2.setBorder(BorderFactory.createTitledBorder(I18N.DIALOG_GALCOL_GALRENDERING));

		galOrientBox=SwingHelper.addComboBox(p2, I18N.DIALOG_GALCOL_GALSEL_ORIENT, new String[] { I18N.ORIENT_VERT, I18N.ORIENT_HORIZ }, 
				 galCol.getHorizGalleriesNav()==0 ? I18N.ORIENT_VERT : I18N.ORIENT_HORIZ, this);
		galsInHorizRowField=SwingHelper.addIntegerField(p2, I18N.DIALOG_GALCOL_GALSINROW, 3, galCol.getGalleriesInHorizRow(), this);
		wrapGalBox=SwingHelper.addCheckBox(p2, I18N.DIALOG_GALCOL_WRAPGALLERIES, galCol.getWrapGalleries()==1, this);
		
		p1.add(p2);
		p1.add(Box.createVerticalStrut(4));
		
		// the initial selection options
		p2=new JPanel();
		p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
		p2.setBorder(BorderFactory.createTitledBorder(I18N.DIALOG_GALCOL_INITIALGAL));
		
		restoreGalBox=SwingHelper.addCheckBox(p2, I18N.DIALOG_GALCOL_RESTORE_GAL, galCol.getDefaultGal()<0, this);
		restoreImgBox=SwingHelper.addCheckBox(p2, I18N.DIALOG_GALCOL_RESTORE_IMG, galCol.getDefaultImg()<0, this);
		defImgField=SwingHelper.addIntegerField(p2, I18N.DIALOG_GALCOL_DEFIMG, 5, galCol.getDefaultImg(), this);
		defImgField.setEnabled(!restoreImgBox.isSelected());
		((JComponent)defImgField.getParent()).setBorder(new EmptyBorder(0, 18, 2, 0));		// make indent

		p1.add(p2);

		add(p1, BorderLayout.NORTH);

		// the galleries
		p1=new JPanel(new BorderLayout(4, 4));
		p1.setBorder(BorderFactory.createTitledBorder(I18N.DIALOG_GALCOL_GALLERIES_GROUP));
		
		p2=new JPanel(new BorderLayout());
		galleryTable=new JTable(this);
		galleryTable.getSelectionModel().addListSelectionListener(this);
		galleryTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		TableCellRenderer	tcr=new GalleryCellRenderer();
		galleryTable.getColumnModel().getColumn(0).setPreferredWidth(600);
		galleryTable.getColumnModel().getColumn(0).setCellRenderer(tcr);
		galleryTable.getColumnModel().getColumn(1).setPreferredWidth(600);
		galleryTable.getColumnModel().getColumn(1).setCellRenderer(tcr);
		galleryTable.getColumnModel().getColumn(2).setPreferredWidth(100);
		galleryTable.addMouseListener(this);
		SwingHelper.enableFileDrop(galleryTable, this);
		
		p2.add(galleryTable.getTableHeader(), BorderLayout.NORTH);
		p2.add(new JScrollPane(galleryTable), BorderLayout.CENTER);
		p1.add(p2, BorderLayout.CENTER);
		
		JToolBar tb=new JToolBar(JToolBar.HORIZONTAL);
		tb.add(ACTION_DIALOG_GAL_ADD_NEW);
		tb.add(ACTION_DIALOG_GAL_ADD_EXISTING);
		tb.add(ACTION_DIALOG_GAL_REMOVE);
		tb.addSeparator();
		tb.add(ACTION_DIALOG_GAL_MOVE_UP);
		tb.add(ACTION_DIALOG_GAL_MOVE_DOWN);
		tb.addSeparator();
		tb.add(ACTION_DIALOG_GAL_OPEN);
		
		p1.add(tb, BorderLayout.NORTH);
		
		add(p1, BorderLayout.CENTER);
		
		// finally add it to the tab
		addToTab();
		
		selectedGalleryChanged();
	}

	public XHTMLPage getProjectPage()
	{
		return (XHTMLPage)getModel();
	}
	
	private void initActions()
	{
		ACTION_DIALOG_GAL_MOVE_UP=new NamedAction(I18N.DIALOG_GALCOL_MOVE_UP)
		{ 
			public void actionPerformed(ActionEvent e) { moveGalleryUp(getSelectedGallery()); } 
		};
		
		ACTION_DIALOG_GAL_MOVE_DOWN=new NamedAction(I18N.DIALOG_GALCOL_MOVE_DOWN)
		{ 
			public void actionPerformed(ActionEvent e) { moveGalleryDown(getSelectedGallery()); } 
		};
		
		ACTION_DIALOG_GAL_REMOVE=new NamedAction(I18N.DIALOG_GALCOL_REMOVE)
		{ 
			public void actionPerformed(ActionEvent e) { removeGallery(getSelectedGallery()); } 
		};

		ACTION_DIALOG_GAL_OPEN=new NamedAction(I18N.DIALOG_GALCOL_OPEN)
		{ 
			public void actionPerformed(ActionEvent e) { openGallery(getSelectedGallery()); } 
		};
		
		ACTION_DIALOG_GAL_ADD_NEW=new NamedAction(I18N.DIALOG_GALCOL_ADD_NEW)
		{ 
			public void actionPerformed(ActionEvent e) { addNewGallery(); } 
		};
		
		ACTION_DIALOG_GAL_ADD_EXISTING=new NamedAction(I18N.DIALOG_GALCOL_ADD_EXISTING)
		{ 
			public void actionPerformed(ActionEvent e) { addExistingGallery(); } 
		};
	}

	private void addPopupActions(List<Action> alList, final Object obj)
	{
		if(obj instanceof XHTMLPanel)
		{
			alList.add(new NamedAction(I18N.DIALOG_GALCOL_ADD_NEW)
				{ 
					public void actionPerformed(ActionEvent e) { addNewGallery(); } 
				});
			alList.add(new NamedAction(I18N.DIALOG_GALCOL_ADD_EXISTING)
				{ 
					public void actionPerformed(ActionEvent e) { addExistingGallery(); } 
				});
		}
		else if(obj instanceof GalleryEntry)
		{
			final GalleryEntry	ge=(GalleryEntry)obj;
			final GalleryPanel	gp=((GalleryEntryUserData)ge.getUserData()).galleryPanel;
			int					index=galCol.indexOf(ge);
			
			alList.add(new NamedAction(I18N.DIALOG_GALCOL_REMOVE, gp==null)
				{ 
					public void actionPerformed(ActionEvent e) { removeGallery(ge); } 
				});
			alList.add(null);	// seperator

			alList.add(new NamedAction(I18N.DIALOG_GALCOL_MOVE_UP, index>0)
				{ 
					public void actionPerformed(ActionEvent e) { moveGalleryUp(ge); } 
				});
			alList.add(new NamedAction(I18N.DIALOG_GALCOL_MOVE_DOWN, index<(galCol.size()-1))
				{ 
					public void actionPerformed(ActionEvent e) { moveGalleryDown(ge); } 
				});
						
			if(gp==null)
			{
				alList.add(null);	// seperator
				alList.add(new NamedAction(I18N.DIALOG_GALCOL_OPEN)
					{ 
						public void actionPerformed(ActionEvent e) { openGallery(ge); } 
					});
			}
		}
	}
	
	private void selectedGalleryChanged()
	{
		int	i=galleryTable.getSelectedRow();

		ACTION_DIALOG_GAL_MOVE_UP.setEnabled(i>0);
		ACTION_DIALOG_GAL_MOVE_DOWN.setEnabled(i>=0 && i<(galCol.size()-1));
		if(i>=0 && i<galCol.size())
		{
			GalleryEntry			ge=galCol.getEntryAt(i);
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			ACTION_DIALOG_GAL_REMOVE.setEnabled(gud.galleryPanel==null);
			ACTION_DIALOG_GAL_OPEN.setEnabled(gud.galleryPanel==null);
		}
		else
		{
			ACTION_DIALOG_GAL_REMOVE.setEnabled(false);
			ACTION_DIALOG_GAL_OPEN.setEnabled(false);
		}
		
		builder.setSelectedObject(this, getSelectedObject());
	}

	public Object getSelectedObject() 
	{
		int	i=galleryTable.getSelectedRow();
		
		if(i>=0 && i<galCol.size())
			return galCol.getEntryAt(i);
		
		return this;
	}

	public void setSelectedObject(Object o) 
	{
		int	index=-1;
		if(o instanceof GalleryEntry)
			index=galCol.indexOf((GalleryEntry)o);
		
		if(galleryTable.getSelectedRow()!=index)
		{
			if(index<0)
				galleryTable.clearSelection();
			else
				galleryTable.setRowSelectionInterval(index, index);
		}
	}

	GalleryEntry getSelectedGallery()
	{
		int	i=galleryTable.getSelectedRow();

		if(i>=0 && i<galCol.size())
			return galCol.getEntryAt(i);
	
		return null;
	}
	
	@Override
	String _getTitle()
	{
		return getProjectPage().getXhtmlFile().getName();
	}

	@Override
	public String getTooltip()
	{
		return getProjectPage().getXhtmlFile().getPath();
	}
	
	@Override
	public Icon getIcon()
	{
		return I18N.ICON_XHTML;
	}
	
	private void updateModel()
	{
		XHTMLPage	page=getProjectPage();
		
		page.setTitle(titleField.getText());
		galCol.setDefaultImg(SwingHelper.getIntValue(defImgField));
		galCol.setHorizGalleriesNav(MiscUtil.equals(galOrientBox.getSelectedItem(), I18N.ORIENT_HORIZ) ? 1 : 0);
		galCol.setGalleriesInHorizRow(SwingHelper.getIntValue(galsInHorizRowField));
		galCol.setWrapGalleries(wrapGalBox.isSelected() ? 1 : 0);
	}

	
	@Override
	public void saveUpdatedModel() throws Exception 
	{
		updateModel();
		
		getProjectPage().save();

		setModified(false);
	}
	
	public void upgradeProject() throws Exception
	{
		updateModel();
		
		getProjectPage().upgrade();
		
		setModified(false);
		
		versionFileField.setText(getProjectPage().getJSGalVersion());
	}

	@Override
	public boolean closePanel() 
	{
		ArrayList<GalleryPanel>	openGals=new ArrayList<GalleryPanel>();
		ArrayList<GalleryPanel>	modifiedGals=new ArrayList<GalleryPanel>();
		ArrayList<File>			modifiedFiles=new ArrayList<File>();

		if(isModified())
			modifiedFiles.add(getProjectPage().getXhtmlFile());
		
		for(int i=0; i<galCol.size(); i++)
		{
			GalleryEntry			ge=galCol.getEntryAt(i);
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			if(gud.galleryPanel!=null)
			{
				if(gud.galleryPanel.isModified())
				{
					modifiedGals.add(gud.galleryPanel);
					modifiedFiles.add(gud.galleryPanel.getGalleryFile());
				}
				else
					openGals.add(gud.galleryPanel);
			}
		}
		
		// do we have unsaved modifications?
		if(modifiedFiles.size()>0)
		{
			Map<File, Boolean>	smap=null;
			
			if(modifiedFiles.size()==1)
			{
				File	f=modifiedFiles.get(0);
				Boolean	b=MessageDialog.getSaveModifiedFile(this, f);
				
				if(b!=null)
				{
					smap=new HashMap<File, Boolean>();
					smap.put(f, b);
				}
			}
			else
				smap=MessageDialog.getSaveModifiedFiles(this, modifiedFiles.toArray(new File[modifiedFiles.size()]));

			if(smap==null)		// cancel
				return false;

			// save or close !!!
			try
			{
				for(GalleryPanel gp: modifiedGals)
				{
					Boolean	b=smap.get(gp.getGalleryFile());
					
					if(b.booleanValue())	// save
						gp.saveUpdatedModel();
					else
						gp.setModified(false);	// mark as clean, so that we can close it
					
					if(!gp.closePanel())
						return false;
				}
				
				if(isModified() && smap.get(getProjectPage().getXhtmlFile()).booleanValue())	// should project be saved?
					saveUpdatedModel();
			}
			catch(Exception e)
			{
				builder.reportProblem(e);
				return false;
			}
		}
			
		// now can close the remaining open galleries
		for(GalleryPanel gp: openGals)
		{
			if(!gp.closePanel())
				return false;
		}
		
		// finally remove also this panel
		builder.removePanel(this);
		
		return true;
	}

	@Override
	public void showPopup(Component source, Object obj, int x, int y) 
	{
		ArrayList<Action>	al=new ArrayList<Action>();
		
		if(obj instanceof IModelPanel)
		{
			builder.getBuilderActions().addPopupActions(al, (IModelPanel)obj);
			al.add(null);	// seperator
		}

		addPopupActions(al, obj);
		
		if(obj instanceof GalleryEntry)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)((GalleryEntry)obj).getUserData();
			
			if(gud.galleryPanel!=null)
			{
				al.add(null);	// seperator
				builder.getBuilderActions().addPopupActions(al, gud.galleryPanel);
				
				al.add(null);	// seperator
				gud.galleryPanel.addPopupActions(al, gud.galleryPanel);
			}
		}

		SwingHelper.showPopup(source, x, y, al);
	}

	// the button actions
	void moveGalleryUp(GalleryEntry	ge)
	{
		if(ge!=null)
		{
			int	i2=galCol.indexOf(ge);
			int	i1=i2-1;
			
			if(i1>=0)
			{
				galCol.removeEntry(i2);
				galCol.addEntry(i1, ge);
				
				fireTableModelEvent(i1, i2, TableModelEvent.UPDATE);
				galleryTable.getSelectionModel().setLeadSelectionIndex(i1);
				
				// finally inform tree of changed structure
				builder.treeObjectChanged(this, true);
			}
		}
	}
	
	void moveGalleryDown(GalleryEntry ge)
	{
		if(ge!=null)
		{
			int	i1=galCol.indexOf(ge);
			int	i2=i1+1;
			
			if(i1<(galCol.size()-1))
			{
				galCol.removeEntry(i1);
				galCol.addEntry(i2, ge);
				
				fireTableModelEvent(i1, i2, TableModelEvent.UPDATE);
				galleryTable.getSelectionModel().setLeadSelectionIndex(i2);
				
				// finally inform tree of changed structure
				builder.treeObjectChanged(this, true);
			}
		}
	}
	
	boolean openGallery(GalleryEntry ge)
	{
		if(ge!=null)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			if(gud.galleryPanel==null)
			{
				try
				{
					Gallery	g=Gallery.parseXML(gud.galleryFile);
					
					// now add the panel
					GalleryPanel	gp=builder.addGalleryPanel(ge, g);
					gud.galleryPanel=gp;
					selectedGalleryChanged();
					
					// finally inform tree of changed structure
					builder.treeObjectChanged(ge, true);
				}
				catch(Exception e)
				{
					builder.reportProblem(e);
					return false;
				}
			}
		}		
		
		return true;
	}
	
	void closeGallery(GalleryEntry ge)
	{
		if(ge!=null)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			if(gud.galleryPanel!=null)
				gud.galleryPanel.closePanel();
		}
	}
	
	protected void openGalleryClosed(GalleryEntry ge)
	{
		if(ge!=null)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			gud.galleryPanel=null;
				
			selectedGalleryChanged();
				
			// finally inform tree of changed structure
			builder.treeObjectChanged(ge, true);
		}
	}
	
	void addNewGallery()
	{
		JPanel		p=new JPanel(new GridLayout(0, 1));
		JCheckBox	autoBox=SwingHelper.addCheckBox(p, I18N.NEW_GALLERY_AUTOSAVE, true);
		JCheckBox	imgBox=SwingHelper.addCheckBox(p, I18N.NEW_GALLERY_ADDIMGS, true);
		String		title=MessageDialog.getInput(this, I18N.NEW_GALLERY_TITLE, I18N.NEW_GALLERY_MSG, null, p);
		
		if(title==null || title.length()<=0)
			return;
		
		File	f;
		
		if(autoBox.isSelected())
		{
			String	fn=MiscUtil.stringToFilename(title)+"."+I18N.EXT_XML; //$NON-NLS-1$
			
			f=MiscUtil.getRelativeFile(getProjectPage().getGalleriesFile(), fn);
			int		i=1;
			while(f.exists())
			{
				f=MiscUtil.getRelativeFile(getProjectPage().getGalleriesFile(), fn+"_"+i+"."+I18N.EXT_XML); //$NON-NLS-1$ //$NON-NLS-2$
				i++;
			}
		}
		else
		{
			f=MiscUtil.getRelativeFile(getProjectPage().getGalleriesFile(), MiscUtil.stringToFilename(title)+'.'+I18N.EXT_XML);
			f=builder.selectFile(f, I18N.XML_FILEFILTER, I18N.NEW_GALLERY_SAVETITLE, I18N.FILE_CHOOSER_SAVE);
		}
		
		if(f!=null)
		{
			try
			{
				if(f.exists())
					throw new IllegalArgumentException(I18NMsg.MSG_GALLERY_FILE_ALREADY_EXISTS.fmt(f.getPath())); 

				String	bn=MiscUtil.getBaseName(f.getName());
				
				Gallery	g=new Gallery(title, bn+"/thumbs", bn+"/images");  //$NON-NLS-1$//$NON-NLS-2$
				
				g.writeXML(f);

				// now create the gallery element and add it
				GalleryEntry			ge=new GalleryEntry(MiscUtil.getRelativePath(getProjectPage().getGalleriesFile(), f), title);
				GalleryEntryUserData	gud=new GalleryEntryUserData(ge, f);
				galCol.addEntry(ge);

				// now add the panel
				GalleryPanel	gp=builder.addGalleryPanel(ge, g);
				gud.galleryPanel=gp;
				selectedGalleryChanged();
				
				int	i=galCol.size()-1;
				fireTableModelEvent(i, i, TableModelEvent.INSERT);
				
				// finally inform tree of changed structure
				builder.treeObjectChanged(this, true);
				
				if(imgBox.isSelected())
				{
					// now open add images dialog
					gp.addImages(null);
				}
			}
			catch(Exception e)
			{
				builder.reportProblem(e);
			}
		}		
	}
	
	void addExistingGallery()
	{
		File	f=builder.selectFile(getProjectPage().getGalleriesFile().getParentFile(), I18N.XML_FILEFILTER, I18N.ADD_GALLERY_OPENTITLE, I18N.FILE_CHOOSER_ADD);
		
		if(f!=null)
		{
			try
			{
				Gallery	g=Gallery.parseXML(f);
				
				// now create the gallery element and add it
				GalleryEntry			ge=new GalleryEntry(MiscUtil.getRelativePath(getProjectPage().getGalleriesFile(), f), g.getTitle());
				GalleryEntryUserData	gud=new GalleryEntryUserData(ge, f);
				
				galCol.addEntry(ge);

				// now add the panel
				GalleryPanel	gp=builder.addGalleryPanel(ge, g);
				gud.galleryPanel=gp;
				selectedGalleryChanged();
				
				int	i=galCol.size()-1;
				fireTableModelEvent(i, i, TableModelEvent.INSERT);
				
				// finally inform tree of changed structure
				builder.treeObjectChanged(this, true);
			}
			catch(Exception e)
			{
				builder.reportProblem(e);
			}
		}		
	}
	
	void removeGallery(GalleryEntry	ge)
	{
		if(ge!=null)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			if(gud!=null && gud.galleryPanel!=null)
			{
				builder.reportError(I18NMsg.MSG_REF_GALLERY_LOADED.msg()); 
			}
			else
			{
				int		i=galCol.indexOf(ge);
				boolean	wasSelected=galleryTable.isRowSelected(i);
				
				galCol.removeEntry(i);
				
				fireTableModelEvent(i, i, TableModelEvent.DELETE);
				
				// finally inform tree of changed structure
				builder.treeObjectChanged(this, true);
				
				// update selection
				if(wasSelected)
				{
					if(i>=galCol.size())
						i=galCol.size()-1;
					
					if(i>=0)
						galleryTable.addRowSelectionInterval(i, i);
					
					selectedGalleryChanged();
				}
			}
		}
	}
	
	// the stuff for the actions we provide
	// the stuff for the galleries collection table model
	@Override
	public void addTableModelListener(TableModelListener l) 
	{
		tablstnrs.add(l);
	}

	@Override
	public void removeTableModelListener(TableModelListener l) 
	{
		tablstnrs.remove(l);
	}

	private void fireTableModelEvent(int firstRow, int lastRow, int type)
	{
		TableModelEvent	e=new TableModelEvent(this, firstRow, lastRow, TableModelEvent.ALL_COLUMNS, type);
		
		for(TableModelListener l: tablstnrs)
			l.tableChanged(e);
		
		setModified(true);
	}
	
	@Override
	public Class<?> getColumnClass(int columnIndex) 
	{
		if(columnIndex==0)
			return String.class;
		else if(columnIndex==1)
			return String.class;
		else if(columnIndex==2)
			return Boolean.class;

		return null;
	}

	@Override
	public int getColumnCount() 
	{
		return 3;
	}

	@Override
	public String getColumnName(int columnIndex) 
	{
		if(columnIndex==0)
			return I18N.COLUMN_GALLERY_FILE;
		else if(columnIndex==1)
			return I18N.COLUMN_GALLERY_TITLE;
		else if(columnIndex==2)
			return I18N.COLUMN_GALLERY_DEFAULT;
		
		return null;
	}

	@Override
	public int getRowCount() 
	{
		return galCol.size();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) 
	{
		GalleryEntry			ge=galCol.getEntryAt(rowIndex);

		// make sure user data is created - just to be sure 
		if(ge.getUserData()==null)
			new GalleryEntryUserData(ge, null);
		
		if(columnIndex==0)
			return ge.getFile();
		else if(columnIndex==1)
			return ge.getTitle();
		else if(columnIndex==2)
			return new Boolean(galCol.getDefaultGal()==rowIndex);

		return null;
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) 
	{
		if(columnIndex==0)
		{
			GalleryEntry			ge=galCol.getEntryAt(rowIndex);
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			return	gud.galleryPanel==null;		// only possible if not yet loaded
		}
		if(columnIndex==2)
			return !restoreGalBox.isSelected();

		return true;
	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) 
	{
		GalleryEntry	ge=galCol.getEntryAt(rowIndex);
		
		if(columnIndex==0)
		{
			ge.setFile((String)value);
			
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			gud.setNewGalleryFile((String)value);
		}
		else if(columnIndex==1)
		{
			ge.setTitle((String)value);
			
			builder.treeObjectChanged(ge, false);
		}			
		else if(columnIndex==2)
		{
			int	oldRow=galCol.getDefaultGal();
			
			galCol.setDefaultGal(rowIndex);
			
			if(oldRow>=0)
				fireTableModelEvent(oldRow, oldRow, TableModelEvent.UPDATE);
		}
		
		setModified(true);
	}
	
	// stuff for listSelectionListener
	@Override
	public void valueChanged(ListSelectionEvent e) 
	{
		selectedGalleryChanged();
	}

	
	// stuff for FileDropListener
	@Override
	public boolean canImport(DropLocation location) 
	{
		return getDropGalleryEntry(location)!=null;
	}

	@Override
	public void filesDropped(DropLocation location, File[] files) 
	{
		GalleryEntry	ge=getDropGalleryEntry(location);
		
		if(ge!=null)
		{
			GalleryEntryUserData	gud=(GalleryEntryUserData)ge.getUserData();
			
			if(gud.galleryPanel==null)
				openGallery(ge);
			
			GalleryPanel	gp=gud.galleryPanel;
			if(gp!=null)
				gp.addImages(files);
		}
	}

	private GalleryEntry getDropGalleryEntry(DropLocation loc)
	{
		if(loc instanceof JTable.DropLocation)
		{
			int	row=((JTable.DropLocation)loc).getRow();
			
			if(row>=0)
				return galCol.getEntryAt(row);
		}
		
		return null;
	}
	
	// stuff for ActionListener
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		if(e.getSource()==restoreGalBox)
		{
			int	oldRow=galCol.getDefaultGal();
			
			galCol.setDefaultGal(restoreGalBox.isSelected() ? -1 : 0);

			if(oldRow<0)
				oldRow=0;
			
			fireTableModelEvent(oldRow, oldRow, TableModelEvent.UPDATE);
		}
		else if(e.getSource()==restoreImgBox)
		{
			galCol.setDefaultImg(restoreImgBox.isSelected() ? -1 : 0);
			
			defImgField.setText(Integer.toString(galCol.getDefaultImg()));
			defImgField.setEnabled(!restoreImgBox.isSelected());
		}
		
		setModified(true);
	}
	
	
	// stuff for mouse listener
	@Override
	public void mouseClicked(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mouseEntered(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mouseExited(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
		checkForPopup(e);
	}

	@Override
	public void mouseReleased(MouseEvent e)	
	{
		checkForPopup(e);
	}

	private void checkForPopup(MouseEvent e)
	{
		if(e.isPopupTrigger() && e.getSource()==galleryTable)
		{
			Point p = new Point(e.getX(), e.getY());
			
			int row = galleryTable.rowAtPoint(p);
			if(row>=0 && row<galCol.size())
			{
				CellEditor	ce=galleryTable.getCellEditor();
				if(ce!=null)
					ce.stopCellEditing();
				
				showPopup(galleryTable, galCol.getEntryAt(row), e.getX(), e.getY());
			}
		}
	}

	// stuff for cell renderer
	class GalleryCellRenderer extends DefaultTableCellRenderer
	{
		@Override
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) 
		{
			Component	comp=super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
			
			String	tip=null;
			if(column==0)
			{
				GalleryEntry	ge=galCol.getEntryAt(row);
				File			f=MiscUtil.getRelativeFile(getProjectPage().getGalleriesFile(), ge.getFile());
				
				try
				{
					tip=f.getCanonicalPath();
				}
				catch(IOException e)
				{
					tip=f.getPath();
				}
			}
			else if(column==1)
			{
				tip=(String)value;
			}
			setToolTipText(tip);
			
			return comp;
		}
	}
	
	class GalleryEntryUserData
	{
		GalleryPanel	galleryPanel;	// if loaded
		File			galleryFile;
		String			galleryPath;
		
		GalleryEntryUserData(GalleryEntry ge, File gf)
		{
			galleryPanel=null;
			if(gf==null)
				setNewGalleryFile(ge.getFile());
			else
			{
				galleryFile=gf;
			
				updateGalleryPath();
			}
			
			ge.setUserData(this);
		}
		
		void setNewGalleryFile(String newFile)
		{
			galleryFile=MiscUtil.getRelativeFile(getProjectPage().getGalleriesFile(), newFile);
			
			updateGalleryPath();
		}
		
		private void updateGalleryPath()
		{
			try
			{
				galleryFile=galleryFile.getCanonicalFile();
			}
			catch(IOException e)
			{
				// we continue with the not canonical file here
			}
			
			galleryPath=galleryFile.getPath();
		}
		
		protected XHTMLPanel getXHTMLPanel()
		{
			return XHTMLPanel.this;
		}
		
		protected GalleryPanel getGalleryPanel()
		{
			return galleryPanel;
		}
		
		protected File getGalleryFile()
		{
			return galleryFile;
		}
		
		protected String getGalleryPath()
		{
			return galleryPath;
		}
	}
}