/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;

public class MiscUtil
{
	public static String stringToFilename(String s)
	{
		StringBuilder	sb=new StringBuilder();
		
		for(int i=0; i<s.length(); i++)
		{
			char	c=s.charAt(i);
			
			if((c>='a' && c<='z') || (c>='A' && c<='Z') || (c>='0' && c<='9') || (c=='_'))
			{
				sb.append(c);
			}
			else if(Character.isWhitespace(c) || c=='.' || c==':')
			{
				sb.append('_');
			}
		}
		
		return sb.toString();
	}
	
	public static String getBaseName(String filename)
	{
		int	i=filename.lastIndexOf('.');
		
		if(i>=0)
			return filename.substring(0, i);
		
		return filename;
	}
	
	public static String changeFileNameExt(String s, String ext)
	{
		// make sure we don't overwrite original file name
		String	fn=s;
		if(!s.endsWith('.'+ext))
		{
			int	i=s.lastIndexOf('.');
			
			if(i>=0)
				fn=s.substring(0, i);
		}
		
		return fn+'.'+ext;
	}
	
	@SuppressWarnings("nls")
	public static File backupFile(File f) throws IOException
	{
		if(f.exists())	// no need to backup not existing files
		{
			File	pf=f.getParentFile();
			String	name=f.getName();
			File	bf;
			int		cnt=0;
			
			// try to find a free backup name
			do
			{
				String	ext="bak";
				
				if(cnt>0)
					ext+=cnt;
				
				bf=new File(pf, changeFileNameExt(name, ext));
				
				cnt++;
			}while(bf.exists());
			
			if(!f.renameTo(bf))
				throw new IOException("failed to rename '"+f.getPath()+"' to '"+bf.getPath()+"'");
			
			return bf;
		}
		
		return null;
	}
	
	public static String getStacktrace(Throwable t)
	{
		StringWriter	sw=new StringWriter();
		PrintWriter		pw=new PrintWriter(sw);
		
		t.printStackTrace(pw);
		
		pw.flush();
		
		return sw.toString();
	}
	
	public static String getRelativePath(File baseFile, File refFile) throws IOException
	{
		File	bfa[]=getParents(baseFile.getCanonicalFile());
		File	rfa[]=getParents(refFile.getCanonicalFile());
		String	name=refFile.getName();

		// now find common parents
		int	i=0;
		while(i<bfa.length && i<rfa.length)
		{
			if(!bfa[i].equals(rfa[i]))
				break;
			
			i++;
		}
		
		// now that we have the common parents we can build the path string
		StringBuilder	sb=new StringBuilder();
		
		for(int j=i; j<bfa.length; j++)		// how many levels do we have to go back
			sb.append("../"); //$NON-NLS-1$
		
		for(int j=i; j<rfa.length; j++)		// now add reffile dirs
			sb.append(rfa[j].getName()+'/');

		// finally add the file name
		sb.append(name);
		
		return sb.toString();
	}
	
	public static File getRelativeFile(File baseFile, String relPath)
	{
		File	base=(baseFile.isDirectory()) ? baseFile : baseFile.getParentFile();
		
		return (relPath.length()<=0) ? base : new File(base, relPath);
	}
	
	private static File[] getParents(File file)
	{
		ArrayList<File>	al=new ArrayList<File>();
	
		File	f=file;
		for(;;)
		{
			f=f.getParentFile();
			
			if(f==null)
				break;
			
			al.add(f);
		}
		
		File	fa[]=new File[al.size()];
		for(int i=al.size()-1, j=0; i>=0; i--, j++)
		{
			fa[j]=al.get(i);
		}
		
		return fa;
	}
	
	public final static boolean equals(Object o1, Object o2)
	{
		if(o1==o2)
			return true;
		else if(o1==null || o2==null)
			return false;
		
		return o1.equals(o2);
	}
	
	public static String getData(InputStream is, String enc) throws IOException
	{
		return new String(getData(is), enc);
	}
	
	public static byte[] getData(InputStream is) throws IOException
	{
		ByteArrayOutputStream	baos=new ByteArrayOutputStream();
		byte					buf[]=new byte[512];
		
		for(;;)
		{
			int	n=is.read(buf);
			
			if(n<0)
				break;
			
			baos.write(buf, 0, n);
		}
		
		return baos.toByteArray();
	}
}